<?php

namespace Drupal\ptools\Utils;

use LogicException;

/**
 * Text-related utility API.
 */
class Text {

  /**
   * Converts ISO-8859-1 data into UTF-8, if needed.
   *
   * @param string|array $input
   *   The input data, which can be a string or an array of strings.
   *
   * @return string|array
   *   The data eventually converted to UTF-8.
   */
  public static function utf8Encode($input) {
    if (is_array($input)) {
      return array_map(__METHOD__, $input);
    }
    elseif (is_string($input)) {
      if (!function_exists('mb_detect_encoding')) {
        throw new LogicException('mbstring extension is required by ' . __METHOD__  . '.');
      }
      // Strict detection (third param) is required.
      $encoding = mb_detect_encoding($input, 'UTF-8', TRUE);
      if ($encoding != 'UTF-8') {
        if (empty($encoding)) {
          // Assume ISO-8859-1 if mb_detect_encoding() didn't work.
          $encoding = 'ISO-8859-1';
        }
        return drupal_convert_to_utf8($input, $encoding);
      }
      else {
        return $input;
      }
    }
    return NULL;
  }

  /**
   * Returns the UTF-16 value of a character.
   *
   * This is the unicode/multibyte equivalent of ord().
   *
   * @param string $u
   *   The character to convert.
   *
   * @return int
   *   Its UTF-16 decimal value.
   */
  public static function uniord($u) {
    $k = mb_convert_encoding($u, 'UCS-2LE', 'UTF-8');
    $k1 = ord(substr($k, 0, 1));
    $k2 = ord(substr($k, 1, 1));
    return $k2 * 256 + $k1;
  }

  /**
   * Returns a valid machine name from a human readable name.
   *
   * @param string $human_name
   *   A human readable string.
   *
   * @return string
   *   The corresponding machine name.
   *
   * @see http://www.makina-corpus.org/blog/code-snippet-human-readable-machine-name
   */
  public static function toMachineName($human_name) {
    $patterns = [
      '/[^a-zA-Z0-9]+/',
      '/-+/',
      '/^-+/',
      '/-+$/',
    ];
    return drupal_strtolower(preg_replace($patterns, ['_', '_', '', ''], $human_name));
  }

  /**
   * Decodes a string from machine name.
   *
   * This implements a best-effort strategy to convert a machine name to a
   * plausible human-readable name, but cannot guarantee full complementarity
   * with ::toMachineName().
   *
   * @param string $string
   *   A machine-name string.
   *
   * @return string
   *   A human readable string.
   */
  public static function fromMachineName($string) {
    return drupal_ucfirst(str_replace('_', ' ', $string));
  }

}
